from flask import Flask, render_template, request, redirect, url_for, session, jsonify
import pandas as pd
import mysql.connector

app = Flask(__name__)
app.secret_key = 'booksesh'

RESULTS_PER_PAGE = 10  


def create_connection():
    return mysql.connector.connect(
        host="localhost",
        user="sayeem",
        password="123456",
        database="bookstore_db",
        sql_mode="NO_ENGINE_SUBSTITUTION"
    )

def create_writers_table(cursor):
    cursor.execute('''
        CREATE TABLE IF NOT EXISTS writers (
            author_name VARCHAR(100),
            isbn VARCHAR(20),
            isbn13 VARCHAR(20),
            UNIQUE KEY unique_writer (author_name, isbn, isbn13)
        )
    ''')

def create_book_table(cursor):
    cursor.execute('''
        CREATE TABLE IF NOT EXISTS book (
            id INT AUTO_INCREMENT PRIMARY KEY,
            title VARCHAR(255) CHARACTER SET utf8mb4,
            average_rating FLOAT,
            isbn VARCHAR(20) UNIQUE,  
            isbn13 VARCHAR(20),
            language_code VARCHAR(10),
            num_pages INT,
            ratings_count INT,
            text_reviews_count INT,
            publication_date VARCHAR(20),
            publisher VARCHAR(100),
            inventory INT,
            price FLOAT
        )
    ''')

def create_cart_table(cursor):
    cursor.execute('''
        CREATE TABLE IF NOT EXISTS cart (
            id INT AUTO_INCREMENT PRIMARY KEY,
            user_email VARCHAR(50),
            isbn VARCHAR(20),
            quantity INT,
            inventory INT,
            FOREIGN KEY (user_email) REFERENCES users(email),
            UNIQUE KEY unique_cart_item (user_email, isbn)
        )
    ''')
    
def create_reviews_table(cursor):
    cursor.execute('''
        CREATE TABLE IF NOT EXISTS reviews (
            id INT AUTO_INCREMENT PRIMARY KEY,
            isbn VARCHAR(20),
            user_email VARCHAR(50),
            review_text TEXT,
            review_time TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
            FOREIGN KEY (isbn) REFERENCES book(isbn),
            FOREIGN KEY (user_email) REFERENCES users(email)
        )
    ''')


def create_order_history_table(cursor):
    cursor.execute('''
        CREATE TABLE IF NOT EXISTS order_history (
            id INT AUTO_INCREMENT PRIMARY KEY,
            user_email VARCHAR(50),
            isbn VARCHAR(20),
            quantity INT,
            price FLOAT,
            total_price FLOAT,
            order_time TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
            FOREIGN KEY (user_email) REFERENCES users(email),
            FOREIGN KEY (isbn) REFERENCES book(isbn)
        )
    ''')



def add_to_cart(user_email, isbn, inventory):
    try:
        with create_connection() as conn:
            with conn.cursor() as cursor:
                create_cart_table(cursor)

                
                book_inventory = get_book_inventory(isbn)

                
                cursor.execute("""
                    SELECT quantity
                    FROM cart
                    WHERE user_email = %s AND isbn = %s
                """, (user_email, isbn))
                cart_item = cursor.fetchone()

                if cart_item:
                    
                    new_quantity = min(cart_item[0] + inventory, book_inventory)
                    cursor.execute("""
                        UPDATE cart SET quantity = %s
                        WHERE user_email = %s AND isbn = %s
                    """, (new_quantity, user_email, isbn))
                else:
                    
                    initial_quantity = min(inventory, book_inventory, 1)
                    cursor.execute("""
                        INSERT INTO cart (user_email, isbn, quantity, inventory)
                        VALUES (%s, %s, %s, %s)
                    """, (user_email, isbn, initial_quantity, inventory))

                conn.commit()
    except Exception as e:
        print(f"Error in add_to_cart: {e}")



def get_book_inventory(isbn):
    
    with create_connection() as conn:
        with conn.cursor() as cursor:
            cursor.execute("SELECT inventory FROM book WHERE isbn = %s", (isbn,))
            result = cursor.fetchone()
            return result[0] if result else 0


def drop_tables(cursor):
    cursor.execute('DROP TABLE IF EXISTS reviews')
    cursor.execute('DROP TABLE IF EXISTS cart')
    cursor.execute('DROP TABLE IF EXISTS book')
    cursor.execute('DROP TABLE IF EXISTS writers')
    cursor.execute('DROP TABLE IF EXISTS users')   
    cursor.execute('DROP TABLE IF EXISTS order_history') 

def populate_database():
    df = pd.read_csv("books.csv")

    
    print("CSV Column Names:", df.columns)

    conn = create_connection()
    cursor = conn.cursor()

    drop_tables(cursor)

    create_users_table(cursor)
    create_book_table(cursor)
    create_writers_table(cursor)
    create_cart_table(cursor)
    create_reviews_table(cursor)
    create_order_history_table(cursor)


    num_pages_column_name = '  num_pages'  
    for _, row in df.iterrows():
        
        row = row.where(pd.notna(row), None)

        
        if num_pages_column_name not in row:
            print(f"Warning: '{num_pages_column_name}' column not found in row {row}")
            continue

        try:
            
            authors = row['authors'].split('/')
            for author in authors:
                cursor.execute("INSERT IGNORE INTO writers (author_name, isbn, isbn13) VALUES (%s, %s, %s)",
                            (author, row['isbn'], row['isbn13']))

            
            cursor.execute(
                "INSERT IGNORE INTO book (title, average_rating, isbn, isbn13, language_code, num_pages, ratings_count, text_reviews_count, publication_date, publisher, inventory, price) VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)",
                (row['title'][:255], row['average_rating'], row['isbn'], row['isbn13'], row['language_code'],
                row['  num_pages'], row['ratings_count'], row['text_reviews_count'], row['publication_date'], row['publisher'], 10, 15.0)
            )
        except KeyError as e:
            print(f"KeyError in row {row}: {e}")
            continue

    conn.commit()  
    cursor.close()
    conn.close()

    return "Database populated successfully!"



def create_users_table(cursor):
    cursor.execute('''
        CREATE TABLE IF NOT EXISTS users (
            id INT AUTO_INCREMENT PRIMARY KEY,
            name VARCHAR(50),
            email VARCHAR(50) UNIQUE,
            password VARCHAR(80),
            role VARCHAR(10)
        )
    ''')


def signup_user(name, email, password, role):
    conn = create_connection()
    cursor = conn.cursor()

    create_users_table(cursor)

    cursor.execute("INSERT INTO users (name, email, password, role) VALUES (%s, %s, %s, %s)", (name, email, password, role))
    
    conn.commit()
    cursor.close()
    conn.close()

def login_user(email, password):
    conn = create_connection()
    cursor = conn.cursor(dictionary=True)

    create_users_table(cursor)

    cursor.execute("SELECT id, name, email, role FROM users WHERE email = %s AND password = %s", (email, password))
    user = cursor.fetchone()

    cursor.close()
    conn.close()

    if user:
        session['role'] = user['role']
        session['loggedin'] = True
        session['user_id'] = user['id']
        session['name'] = user['name']
        session['email'] = user['email']

    return user


@app.route('/populate_database')
def populate():
    return populate_database()

@app.route('/')
def index():
    featured_books = get_featured_books()
    return render_template('index.html', active_section='home', featured_books=featured_books)

def get_featured_books():
    try:
        with create_connection() as conn:
            with conn.cursor(dictionary=True) as cursor:
                cursor.execute("SELECT title, isbn, publisher FROM book ORDER BY average_rating DESC LIMIT 5")
                return cursor.fetchall()
    except Exception as e:
        print(f"Error fetching featured books: {e}")
        return []



@app.route('/account')
def account():
    return render_template('account.html', active_section="account")

@app.route('/signup', methods=['GET', 'POST'])
def signup():
    if request.method == 'POST':
        userDetails = request.form
        name = userDetails['name']
        email = userDetails['email']
        password = userDetails['password']
        role = userDetails['role']  

        # Check if user already exists
        if check_user_exists(email):
            return render_template('account.html', error="Email already registered!")

        # Attempt to signup user
        try:
            if signup_user(name, email, password, role):
                session['loggedin'] = True
                session['email'] = email
                return redirect(url_for('profile'))
            else:
                return render_template('account.html', error="An error occurred during sign-up.")
        except Exception as e:
            print(f"Signup error: {e}")
            return render_template('account.html', error="An error occurred during sign-up.")

    return render_template('account.html')

def check_user_exists(email):
    try:
        conn = create_connection()
        cursor = conn.cursor(dictionary=True)
        cursor.execute("SELECT id FROM users WHERE email = %s", (email,))
        user = cursor.fetchone()
        cursor.close()
        conn.close()
        return user is not None
    except Exception as e:
        print(f"Error checking user existence: {e}")
        return False


@app.route('/login', methods=['GET', 'POST'])
def login():
    msg = ''
    if request.method == 'POST':
        email = request.form['email']
        password = request.form['password']
        user = login_user(email, password)
        if user:
            session['loggedin'] = True
            session['email'] = user['email']
            return redirect(url_for('profile'))
        else:
            msg = 'Incorrect username/password!'
    return render_template('account.html', msg=msg)

@app.route('/logout')
def logout():
    session.pop('loggedin', None)  
    session.pop('email', None)  
    session.clear()
    return redirect(url_for('login'))  

@app.route('/profile')
def profile():
    if 'loggedin' in session:
        user_email = session['email']
        order_history = []
        user_reviews = []
        

        with create_connection() as conn:
            with conn.cursor(dictionary=True) as cursor:
                # Fetch order history
                cursor.execute("""
                    SELECT * FROM order_history
                    WHERE user_email = %s
                    ORDER BY order_time DESC
                """, (user_email,))
                order_history = cursor.fetchall()

                # Fetch user reviews
                cursor.execute("""
                    SELECT r.*, b.title, b.publisher, b.average_rating
                    FROM reviews r
                    JOIN book b ON r.isbn = b.isbn
                    WHERE r.user_email = %s
                    ORDER BY r.review_time DESC
                """, (user_email,))
                user_reviews = cursor.fetchall()

        if session['role'] == 'manager':
            users = get_all_users()
            return render_template('manager_profile.html', users=users, order_history=order_history)
        else:
            return render_template('profile.html', order_history=order_history, user_reviews=user_reviews)
    return redirect(url_for('login'))


# ...

@app.route('/promote_user', methods=['POST'])
def promote_user():
    if 'loggedin' in session and session['role'] == 'manager':
        user_id = request.form['user_id']
        try:
            with create_connection() as conn:
                with conn.cursor() as cursor:
                    cursor.execute("UPDATE users SET role = 'manager' WHERE id = %s", (user_id,))
                    conn.commit()
                    return redirect(url_for('profile'))
        except Exception as e:
            print(f"Error promoting user: {e}")
            return redirect(url_for('profile'))
    return redirect(url_for('login'))


@app.route('/browse', methods=['GET', 'POST'])
def browse():
    try:
        conn = create_connection()
        cursor = conn.cursor(dictionary=True)

        create_book_table(cursor)
        create_writers_table(cursor)

       
        page = int(request.args.get('page', 1))

        
        offset = (page - 1) * RESULTS_PER_PAGE

        
        search_term = request.form.get('search', '')

        
        author_query = """
            SELECT b.*, w.author_name, b.average_rating
            FROM book b
            LEFT JOIN writers w ON b.isbn = w.isbn OR b.isbn13 = w.isbn13
            WHERE LOWER(w.author_name) LIKE %s
            OR LOWER(b.title) LIKE %s
            OR LOWER(b.publisher) LIKE %s
            OR b.isbn LIKE %s
            OR b.isbn13 LIKE %s
            ORDER BY b.id ASC
            LIMIT %s, %s;
        """

        # Print the SQL query for debugging
        print("SQL Query:", author_query % (f'%{search_term.lower()}%', f'%{search_term.lower()}%', f'%{search_term.lower()}%', f'%{search_term}%', f'%{search_term}%', offset, RESULTS_PER_PAGE))

        
        cursor.execute(author_query, (f'%{search_term.lower()}%', f'%{search_term.lower()}%', f'%{search_term.lower()}%', f'%{search_term}%', f'%{search_term}%', offset, RESULTS_PER_PAGE))
        results = cursor.fetchall()

        
        print("Query Results:", results)

        # Close the database connection
        cursor.close()
        conn.close()

        
        for result in results:
            
            if result['author_name']:
                result['authors'] = [{'author_name': author.strip()} for author in result['author_name'].split(',')]
                result['title_initial'] = result['title'][0] if result['title'] else ''
            else:
                result['authors'] = []

        
        total_pages = (len(results) + RESULTS_PER_PAGE - 1) // RESULTS_PER_PAGE

        return render_template('browse.html', results=results, page=page, total_pages=total_pages, active_section='browse')
    except Exception as e:
        return jsonify({"error": str(e)})

    
    
@app.route('/add_to_cart', methods=['POST'])
def add_to_cart_route():
    try:
        if 'loggedin' in session:
            user_email = session['email']
            isbn = request.form['isbn']
            inventory = int(request.form['inventory'])
            add_to_cart(user_email, isbn, inventory)
            return jsonify({"success": True, "message": "Item added to the cart successfully."})
        else:
            return jsonify({"success": False, "message": "User not logged in."})
    except Exception as e:
        return jsonify({"success": False, "message": str(e)})

@app.route('/cart')
def cart():
    try:
        if 'loggedin' in session:
            user_email = session['email']

            conn = create_connection()
            cursor = conn.cursor(dictionary=True)

            create_cart_table(cursor)

            
            cursor.execute("""
                SELECT c.*, b.average_rating, b.inventory - c.quantity AS available_inventory
                FROM cart c
                JOIN book b ON c.isbn = b.isbn
                WHERE c.user_email = %s
            """, (user_email,))
            cart_items = cursor.fetchall()

            cursor.close()
            conn.close()

            return render_template('cart.html', cart_items=cart_items, active_section='cart')
        else:
            return redirect(url_for('login'))
    except Exception as e:
        return jsonify({"error": str(e)})


@app.route('/update_cart_quantity', methods=['POST'])
def update_cart_quantity():
    try:
        if 'loggedin' in session:
            user_email = session['email']
            isbn = request.form['isbn']
            quantity = int(request.form['quantity'])

            
            current_inventory = get_book_inventory(isbn)

            
            update_cart_quantity_in_database(user_email, isbn, quantity, current_inventory)

            
            new_inventory = current_inventory - quantity

            
            return jsonify({"success": True, "new_inventory": new_inventory})
        else:
            return jsonify({"success": False, "message": "User not logged in."})
    except Exception as e:
        return jsonify({"success": False, "message": str(e)})



def update_cart_quantity_in_database(user_email, isbn, quantity, inventory):
    try:
        with create_connection() as conn:
            with conn.cursor() as cursor:
                # Update the quantity in the cart table
                cursor.execute("""
                    UPDATE cart SET quantity = %s, inventory = %s
                    WHERE user_email = %s AND isbn = %s
                """, (quantity, inventory, user_email, isbn))

                conn.commit()
    except Exception as e:
        print(f"Error in update_cart_quantity_in_database: {e}")


@app.route('/checkout', methods=['GET'])
def checkout():
    if 'loggedin' in session:
        user_email = session['email']

        with create_connection() as conn:
            with conn.cursor(dictionary=True) as cursor:
                
                cursor.execute("""
                    SELECT c.*, b.title, b.price
                    FROM cart c
                    JOIN book b ON c.isbn = b.isbn
                    WHERE c.user_email = %s
                """, (user_email,))
                cart_items = cursor.fetchall()

        
        total_order_price = sum(item['quantity'] * item['price'] for item in cart_items)

        
        with create_connection() as conn:
            with conn.cursor() as cursor:
                create_order_history_table(cursor)
                for item in cart_items:
                    cursor.execute("""
                        INSERT INTO order_history (user_email, isbn, quantity, price, total_price)
                        VALUES (%s, %s, %s, %s, %s)
                    """, (user_email, item['isbn'], item['quantity'], item['price'], total_order_price))
            conn.commit()

        return render_template('checkout_receipt.html', cart_items=cart_items, total_price=total_order_price)
    else:
        return redirect(url_for('login'))



@app.route('/update_rating', methods=['POST'])
def update_rating():
    if 'loggedin' in session:
        try:
            data = request.json
            isbn = data['isbn']
            rating = data['rating']

            
            new_avg_rating = update_book_rating(isbn, rating)

            
            return jsonify({"success": True, "message": "Rating updated successfully.", "new_avg_rating": new_avg_rating})
        except Exception as e:
            error_message = str(e)
            print(f"Error updating rating: {error_message}")
            return jsonify({"success": False, "message": f"An error occurred: {error_message}"})
    else:
        return jsonify({"success": False, "message": "User not logged in."})


def update_book_rating(isbn, user_rating):
    try:
        with create_connection() as conn:
            with conn.cursor() as cursor:
                
                cursor.execute("SELECT average_rating, ratings_count FROM book WHERE isbn = %s", (isbn,))
                book = cursor.fetchone()
                print("Book Data:", book)

                if book:
                    current_avg_rating, ratings_count = book

                    
                    current_avg_rating = float(current_avg_rating) if current_avg_rating is not None else 0.0
                    ratings_count = int(ratings_count) if ratings_count is not None else 0
                    user_rating = float(user_rating)

                    
                    new_avg_rating = ((current_avg_rating * ratings_count) + user_rating) / (ratings_count + 1)
                    print("New Average Rating:", new_avg_rating)

                    
                    cursor.execute("UPDATE book SET average_rating = %s, ratings_count = ratings_count + 1 WHERE isbn = %s", (new_avg_rating, isbn))
                    conn.commit()

                    return new_avg_rating
    except Exception as e:
        print(f"Error updating book rating: {e}")
        return None  

@app.route('/get_book_details')
def get_book_details():
    isbn = request.args.get('isbn')
    try:
        with create_connection() as conn:
            with conn.cursor(dictionary=True) as cursor:
                cursor.execute("SELECT * FROM book WHERE isbn = %s", (isbn,))
                book_details = cursor.fetchone()
                if book_details:
                    return jsonify(book_details)
                else:
                    return jsonify({"error": "Book not found"}), 404
    except Exception as e:
        return jsonify({"error": str(e)}), 500

@app.route('/book/<isbn>')
def book_details(isbn):
    conn = create_connection()
    cursor = conn.cursor(dictionary=True)

    
    cursor.execute("SELECT * FROM book WHERE isbn = %s", (isbn,))
    book = cursor.fetchone()

    
    cursor.execute("SELECT * FROM reviews WHERE isbn = %s", (isbn,))
    reviews = cursor.fetchall()

    cursor.close()
    conn.close()

    return render_template('book_details.html', book=book, reviews=reviews)



@app.route('/submit_review', methods=['POST'])
def submit_review():
    if 'loggedin' in session:
        user_email = session['email']
        isbn = request.form['isbn']
        review_text = request.form['review_text']
        
        try:
            with create_connection() as conn:
                with conn.cursor() as cursor:
                    cursor.execute("""
                        INSERT INTO reviews (isbn, user_email, review_text)
                        VALUES (%s, %s, %s)
                    """, (isbn, user_email, review_text))
                    conn.commit()
            return redirect(url_for('book_details', isbn=isbn))
        except Exception as e:
            return jsonify({"success": False, "message": str(e)})
    else:
        return jsonify({"success": False, "message": "User not logged in."})


def get_all_users():
    try:
        with create_connection() as conn:
            with conn.cursor(dictionary=True) as cursor:
                cursor.execute("SELECT id, name, email, role FROM users")
                return cursor.fetchall()
    except Exception as e:
        print(f"Error fetching users: {e}")
        return []

@app.route('/view_user_orders/<user_email>')
def view_user_orders(user_email):
    if 'loggedin' in session and session['role'] == 'manager':
        # Fetch order history for the specified user
        order_history = []
        with create_connection() as conn:
            with conn.cursor(dictionary=True) as cursor:
                cursor.execute("""
                    SELECT * FROM order_history
                    WHERE user_email = %s
                    ORDER BY order_time DESC
                """, (user_email,))
                order_history = cursor.fetchall()

        return render_template('user_orders.html', order_history=order_history, user_email=user_email)
    else:
        return redirect(url_for('login'))

@app.route('/view_user_reviews/<user_email>')
def view_user_reviews(user_email):
    if 'loggedin' in session and session['role'] == 'manager':
        # Fetch reviews for the specified user
        user_reviews = []
        with create_connection() as conn:
            with conn.cursor(dictionary=True) as cursor:
                cursor.execute("""
                    SELECT r.*, b.title, b.publisher, b.average_rating
                    FROM reviews r
                    JOIN book b ON r.isbn = b.isbn
                    WHERE r.user_email = %s
                    ORDER BY r.review_time DESC
                """, (user_email,))
                user_reviews = cursor.fetchall()

        return render_template('user_reviews.html', user_reviews=user_reviews, user_email=user_email)
    else:
        return redirect(url_for('login'))

@app.route('/add_book', methods=['POST'])
def add_book():
    if 'loggedin' in session and session['role'] == 'manager':
        
        title = request.form.get('title')
        isbn = request.form.get('isbn')
        isbn13 = request.form.get('isbn13')
        language_code = request.form.get('language_code')
        num_pages = request.form.get('num_pages')
        publication_date = request.form.get('publication_date')
        publisher = request.form.get('publisher')
        inventory = request.form.get('inventory')
        price = request.form.get('price')

       
        if not all([title, isbn, isbn13, language_code, num_pages, publication_date, publisher, inventory, price]):
            return redirect(url_for('profile', error="All fields are required."))

        try:
            with create_connection() as conn:
                with conn.cursor() as cursor:
                    
                    cursor.execute("""
                        INSERT INTO book (title, isbn, isbn13, language_code, num_pages, publication_date, publisher, inventory, price)
                        VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s)
                    """, (title, isbn, isbn13, language_code, num_pages, publication_date, publisher, inventory, price))
                    conn.commit()
            return redirect(url_for('profile'))
        except Exception as e:
            print(f"Error adding book: {e}")
            return redirect(url_for('profile', error="Failed to add book."))
    else:
        return redirect(url_for('login'))


@app.route('/edit_book/<isbn>', methods=['GET', 'POST'])
def edit_book(isbn):
    if 'loggedin' in session and session['role'] == 'manager':
        if request.method == 'POST':
            
            new_inventory = request.form.get('inventory')
            new_price = request.form.get('price')

            if not all([new_inventory, new_price]):
                return redirect(url_for('edit_book', isbn=isbn, error="Inventory and Price are required."))

            try:
                with create_connection() as conn:
                    with conn.cursor() as cursor:
                        
                        cursor.execute("""
                            UPDATE book 
                            SET inventory = %s, price = %s 
                            WHERE isbn = %s
                        """, (new_inventory, new_price, isbn))
                        conn.commit()
                return redirect(url_for('profile', message="Book updated successfully."))
            except Exception as e:
                print(f"Error updating book: {e}")
                return redirect(url_for('edit_book', isbn=isbn, error=f"Failed to update book: {e}"))
        else:
            
            book_data = get_book_data(isbn)
            return render_template('edit_book.html', book=book_data, error=request.args.get('error'))
    else:
        return redirect(url_for('login'))

def get_book_data(isbn):
    
    try:
        with create_connection() as conn:
            with conn.cursor(dictionary=True) as cursor:
                cursor.execute("SELECT * FROM book WHERE isbn = %s", (isbn,))
                return cursor.fetchone()
    except Exception as e:
        print(f"Error fetching book data: {e}")
        return None






@app.route('/delete_book')
def delete_book():
    if 'loggedin' in session and session['role'] == 'manager':
        isbn = request.args.get('isbn')
        author = request.args.get('author')

        try:
            with create_connection() as conn:
                with conn.cursor() as cursor:
                    # Delete the book and its writer details from the database
                    cursor.execute("DELETE FROM book WHERE isbn = %s", (isbn,))
                    cursor.execute("DELETE FROM writers WHERE isbn = %s AND author_name = %s", (isbn, author))
                    conn.commit()
            return redirect(url_for('profile', message="Book deleted successfully."))
        except Exception as e:
            print(f"Error deleting book: {e}")
            return redirect(url_for('profile', error="Failed to delete book."))
    else:
        return redirect(url_for('login'))


@app.route('/search_books', methods=['POST'])
def search_books():
    if 'loggedin' in session and session['role'] == 'manager':
        isbn = request.form.get('isbn', '')
        author = request.form.get('author', '')

        if isbn:
            # Search primarily by ISBN
            query = """
                SELECT DISTINCT b.*, w.author_name FROM book b
                LEFT JOIN writers w ON b.isbn = w.isbn OR b.isbn13 = w.isbn13
                WHERE b.isbn = %s
            """
            search_param = isbn
        else:
         
            query = """
                SELECT DISTINCT b.*, w.author_name FROM book b
                LEFT JOIN writers w ON b.isbn = w.isbn OR b.isbn13 = w.isbn13
                WHERE w.author_name LIKE %s
            """
            search_param = f'%{author}%'

        with create_connection() as conn:
            with conn.cursor(dictionary=True) as cursor:
                cursor.execute(query, (search_param,))
                books = cursor.fetchall()

        return render_template('manager_profile.html', books=books)
    else:
        return redirect(url_for('login'))

@app.route('/update_book/<isbn>', methods=['POST'])
def update_book(isbn):
    if 'loggedin' in session and session['role'] == 'manager':
      
        new_title = request.form.get('title')
        new_isbn13 = request.form.get('isbn13')
        new_language_code = request.form.get('language_code')
        new_num_pages = request.form.get('num_pages')
        new_publication_date = request.form.get('publication_date')
        new_publisher = request.form.get('publisher')
        new_inventory = request.form.get('inventory')
        new_price = request.form.get('price')

     
        if not all([new_title, new_isbn13, new_language_code, new_num_pages, new_publication_date, new_publisher, new_inventory, new_price]):
           
            return redirect(url_for('edit_book', isbn=isbn, error="All fields are required."))

        try:
            with create_connection() as conn:
                with conn.cursor() as cursor:
                 
                    cursor.execute("""
                        UPDATE book
                        SET title = %s, isbn13 = %s, language_code = %s, num_pages = %s, publication_date = %s, publisher = %s, inventory = %s, price = %s
                        WHERE isbn = %s
                    """, (new_title, new_isbn13, new_language_code, new_num_pages, new_publication_date, new_publisher, new_inventory, new_price, isbn))
                    conn.commit()

            return redirect(url_for('profile', message="Book updated successfully."))
        except Exception as e:
            print(f"Error updating book: {e}")
            return redirect(url_for('edit_book', isbn=isbn, error="Failed to update book."))
    else:
        return redirect(url_for('login'))


if __name__ == '__main__':
    app.run(debug=True)